# import torch
import pandas as pd

from fs.encode1 import ENAC2, binary, NCP, EIIP, CKSNAP, ENAC, Kmer, NAC, PseEIIP, ANF, CKSNAP8, Kmer4, TNC, RCKmer5, \
    DNC
from fs.test1 import ACC_encoding, TAC
from fs.test2 import pse, SCPseTNC, PCPseTNC


class Config(object):
    def __init__(self):
        self.model_name = "" # dataset name
        self.is_feature_selection = True
        self.load_global_pretrain_model = False

        self.output_save_path = "M:/data/20230305/"
        self.global_model_save_path = None
        # self.global_model_save_path = "data/global_keras_for_6mA.hdf5"
        # self.model_save_path = self.output_save_path+self.model_name+".h5"
        self.model_save_path = "M:/data/20220720/"
        # self.stacking_model_save_path = "D:/temp/20220720/stacking_model/"
        self.stacking_model_save_path = self.output_save_path+"stacking_model/"
        self.load_best_params = False
        self.data_argument = False
        self.au_re_len = 3

        self.base_models_name = None
        self.base_models_name = ["RF", "AB", "LD", "ET", "GB", "XGB", "LGBM", "LR"]
        # self.base_models_name = ["RF","GNB", "AB", "LD", "ET", "GB", "XGB", "LGBM", "LR"]


        # self.encoding = {"ENAC2": ENAC2, "binary": binary, "NCP": NCP, "EIIP": EIIP}
        # self.encoding = {"NCP": NCP}
        # self.encoding = {'pse':pse,'ACC':ACC_encoding,"ANF":ANF,"binary":binary,"CKSNAP":CKSNAP,"NCP":NCP,"EIIP":EIIP,"ENAC":ENAC,"Kmer":Kmer,"NAC":NAC,"PseEIIP":PseEIIP}
        # self.encoding = {"ANF":ANF,"binary":binary,"CKSNAP":CKSNAP,"NCP":NCP,"EIIP":EIIP,"ENAC":ENAC,"Kmer":Kmer,"NAC":NAC,"PseEIIP":PseEIIP}
        # self.encoding = {"ANF":ANF,"binary":binary,"CKSNAP":CKSNAP,"NCP":NCP,"ENAC":ENAC,"Kmer":Kmer,"NAC":NAC}
        self.encoding = {"ENAC": ENAC2, "binary": binary, "NCP": NCP, "EIIP": EIIP, "Kmer4": Kmer4, "CKSNAP": CKSNAP8,
                     "PseEIIP": PseEIIP, "TNC": TNC, "RCKmer5": RCKmer5, "SCPseTNC": SCPseTNC, "PCPseTNC": PCPseTNC,
                     "ANF": ANF, "NAC": NAC, "TAC": TAC}
        self.dropout = 0
        self.require_improvement = 1000  # early stopping

        self.k = 1
        self.n_vocab = 0  # size of vocab
        self.num_epochs = 500  # epoch
        self.patience = 50
        self.batch_size = 256  # mini-batch
        self.learning_rate = 0.01
        self.encoding_data_save_path = self.output_save_path+"ML_encoding/"
        self.features_save_path = self.output_save_path+"features/"  # the number and type of selected feature
        self.fig_save_path = self.output_save_path+"fig/"
        #-------------------------------------
        self.input_length = 41
        self.num_filters = 80
        self.n_folds = 5
        self.params_all = None
        #-----------------reviewer----------------------------------
        self.reviewer_remove_feature = ""

        # self.params_all = self.get_param2()
        # if self.load_best_params:
        #
        # else:
        #     self.params_all = None

    def update_param(self):
        self.params_all = self.get_param2()

    def __copy__(self):
        tmp = Config()
        tmp.__dict__.update(self.__dict__)
        return tmp


    def get_param2(self):
        datasets =  ["6mA_R.chinensis", "6mA_A.thaliana", "6mA_C.elegans", "6mA_C.equisetifolia",
                    "6mA_D.melanogaster", "6mA_F.vesca", "6mA_H.sapiens", "6mA_S.cerevisiae",
                    "6mA_Tolypocladium", "6mA_Xoc BLS256","4mC_E.coli2020", "4mC_G.pickeringii2020", "4mC_G.subterraneus2020", "4mC_C.equisetifolia",
                    "4mC_F.vesca",
                    "4mC_S.cerevisiae", "4mC_Tolypocladium", "4mC_E.coli", "4mC_G.pickeringii",
                    "4mC_G.subterraneus", "5hmC_H.sapiens", "5hmC_M.musculus","6mA_T.thermophile", "4mC_D.melanogaster",
                     "4mC_A.thaliana", "4mC_C.elegans", "4mC_A.thaliana2020",
                     "4mC_C.elegans2020", "4mC_D.melanogaster2020"]

        # params = pd.read_csv("config/config_params.csv", header=0, index_col=0,sep="	")
        params = pd.read_table("config/param7.txt",header=0,index_col=0)

        data1 = dict()
        for dataset in datasets:
            data1[dataset] = {
                    "param_RF" : {"n_estimators": int(params.loc[dataset,"RF_n_estimators"]) , "max_depth": int(params.loc[dataset,"RF_max_depth"]) },
                    "param_GB" : {"n_estimators": int(params.loc[dataset,"GB_n_estimators"]) , "max_depth": int(params.loc[dataset,"GB_max_depth"]) },
                    "param_AB" : {"n_estimators": int(params.loc[dataset,"AB_n_estimators"]) , "learning_rate": params.loc[dataset,"AB_learning_rate"] },
                    "param_ET" : {"n_estimators": int(params.loc[dataset,"ET_n_estimators"]) , "max_depth": int(params.loc[dataset,"ET_max_depth"]) },
                    "param_XGB" : {"n_estimators": int(params.loc[dataset,"XGB_n_estimators"]) , "max_depth": int(params.loc[dataset,"XGB_max_depth"]), "use_label_encoder": False, "eval_metric": 'logloss'},
                    "param_LR" : {"C": float(params.loc[dataset,"LR_C"])},
                    "param_LD" : {},
                    "param_LGBM" : {"n_estimators":int(params.loc[dataset,"LGBM_n_estimators"]) , "learning_rate": params.loc[dataset,"LGBM_learning_rate"]},
                    "param_GNB":{"var_smoothing":params.loc[dataset,"GNB_var_smoothing"]}
            }
        return data1

    def get_param(self):
        data1 = {
            "6mA_R.chinensis":{
                    "param_RF" : {"n_estimators": 280, "max_depth": 14},
                    "param_GB" : {"n_estimators": 120, "max_depth": 1},
                    "param_AB" : {"n_estimators": 220,"learning_rate":0.1},
                    "param_ET" : {"n_estimators": 260, "max_depth": 15},
                    "param_XGB" : {"n_estimators": 60, "max_depth": 1, "use_label_encoder": False, "eval_metric": 'logloss'},
                    "param_LR" : {"C": 0.001},
                    "param_LD" : {},
                    "param_LGBM" : {"n_estimators": 120, "learning_rate": 0.1},
                    "param_GNB":{"var_smoothing":0.1}
            },

            "6mA_A.thaliana": {
                "param_RF": {"n_estimators": 240, "max_depth": 14},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220,"learning_rate":1},
                "param_ET": {"n_estimators": 220, "max_depth": 14},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {"n_estimators": 240, "learning_rate": 0.1},
                "param_GNB": {"var_smoothing": 0.01}
            },

            "6mA_C.elegans": {
                "param_RF": {"n_estimators": 240, "max_depth": 14},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220,"learning_rate":1},
                "param_ET": {"n_estimators": 120, "max_depth": 14},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {"n_estimators": 240, "learning_rate": 0.1},
                "param_GNB": {"var_smoothing": 0.01}
            },

            "6mA_C.equisetifolia": {
                "param_RF": {"n_estimators": 240, "max_depth": 12},
                "param_GB": {"n_estimators": 100, "max_depth": 4},
                "param_AB": {"n_estimators": 160,"learning_rate":1},
                "param_ET": {"n_estimators": 220, "max_depth": 14},
                "param_XGB": {"n_estimators": 20, "max_depth": 4, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 0.001},
                "param_LD": {},
                "param_LGBM": {"n_estimators": 280, "learning_rate": 0.01},
                "param_GNB": {"var_smoothing": 0.01}
            },

            "6mA_D.melanogaster": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "6mA_F.vesca": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "6mA_H.sapiens": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "6mA_S.cerevisiae": {
                "param_RF": {"n_estimators": 260, "max_depth": 15},
                "param_GB": {"n_estimators": 160, "max_depth": 4},
                "param_AB": {"n_estimators": 80,"learning_rate":1},
                "param_ET": {"n_estimators": 240, "max_depth": 15},
                "param_XGB": {"n_estimators": 360, "max_depth": 10, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 0.1},
                "param_LD": {},
                "param_LGBM": {"n_estimators": 220, "learning_rate": 0.1},
                "param_GNB": {"var_smoothing": 0.1}
            },

            "6mA_Tolypocladium": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "6mA_Xoc BLS256": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_E.coli2020": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_G.pickeringii2020": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_G.subterraneus2020": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_C.equisetifolia": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_F.vesca": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_S.cerevisiae": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_Tolypocladium": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_E.coli": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_G.pickeringii": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_G.subterraneus": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "5hmC_H.sapiens": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "5hmC_M.musculus": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "6mA_T.thermophile": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_D.melanogaster": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_A.thaliana": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_C.elegans": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_A.thaliana2020": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_C.elegans2020": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },

            "4mC_D.melanogaster2020": {
                "param_RF": {"n_estimators": 180, "max_depth": 10},
                "param_GB": {"n_estimators": 260, "max_depth": 5},
                "param_AB": {"n_estimators": 220},
                "param_ET": {},
                "param_XGB": {"n_estimators": 380, "max_depth": 7, "use_label_encoder": False, "eval_metric": 'logloss'},
                "param_LR": {"C": 1},
                "param_LD": {},
                "param_LGBM": {},
            },


        }
        return data1


