import random
import time
from collections import Counter

from lightgbm import LGBMClassifier
from tensorflow.python.keras import Sequential, Input, Model
from tensorflow.python.keras.layers import LeakyReLU, Dense, Dropout, BatchNormalization
from tqdm import tqdm
import numpy as np
from tensorflow.keras.optimizers import Adam
import pandas as pd
import pickle as pkl
from data.data import get_single_data, get_single_data_ac4c
from data.negative_sampling import negative_sampling
from fs.encode1 import ENAC2, binary
from prepare.prepare_ml import ml_code
from tools.tools import get_label, score2, set_global_determinism, plot_tsne2, plot_tsne2_4

#这个版本对生成器和判别器标签进行干扰
dict1 = {
    0:'A',
    1:'C',
    2:'G',
    3:'T',
}

def one_hot(seq):

    nrows = len(seq)
    seq_len = len(seq[0])

    seq_01 = np.zeros((nrows, seq_len, 4), dtype='int')


    for i in range(nrows):
        one_seq = seq[i]
        one_seq = one_seq.replace('A', '0')
        one_seq = one_seq.replace('C', '1')
        one_seq = one_seq.replace('G', '2')
        one_seq = one_seq.replace('U', '3')
        one_seq = one_seq.replace('T', '3')
        seq_start = 0
        for j in range(seq_len):
            if j < seq_start:
                seq_01[i, j, :] = 0.25
            else:
                try:
                    seq_01[i, j, int(one_seq[j - seq_start])] = 1
                except:
                    seq_01[i, j, :] = 0.25
    return seq_01

def load_data(dataset):
    X_train, X_test = get_single_data(dataset)
    Y_train = get_label(X_train)
    X_train = X_train[Y_train == 1]
    X_train = one_hot(X_train.iloc[:, 1].values)
    # Convert shape from (60000, 201, 4) to (60000, 164)
    X_train = X_train.reshape(-1, 164)
    return (X_train, Y_train)


def build_generator(seq_len=201):
    model = Sequential()
    model.add(Dense(units=256, input_dim=100))
    model.add(LeakyReLU(alpha=0.2))
    model.add(BatchNormalization(momentum=0.8))

    model.add(Dense(units=512))
    model.add(LeakyReLU(alpha=0.2))
    model.add(BatchNormalization(momentum=0.8))

    model.add(Dense(units=1024))
    model.add(LeakyReLU(alpha=0.2))
    model.add(BatchNormalization(momentum=0.8))

    model.add(Dense(units=seq_len * 4, activation='tanh'))
    model.compile(loss='binary_crossentropy', optimizer=Adam(0.0002, 0.5))
    return model


def build_discriminator(seq_len=201):
    model = Sequential()
    model.add(Dense(units=1024, input_dim=seq_len*4))
    model.add(LeakyReLU(alpha=0.2))
    model.add(Dropout(0.3))
    model.add(Dense(units=512))
    model.add(LeakyReLU(alpha=0.2))
    model.add(Dropout(0.3))
    model.add(Dense(units=256))
    model.add(LeakyReLU(alpha=0.2))
    model.add(Dropout(0.3))
    model.add(Dense(units=1, activation='sigmoid'))
    model.compile(loss='binary_crossentropy', optimizer=Adam(0.0002, 0.5))
    return model

def build_GAN(discriminator, generator):
    discriminator.trainable=False
    GAN_input = Input(shape=(100,))
    x = generator(GAN_input)
    GAN_output= discriminator(x)
    GAN = Model(inputs=GAN_input, outputs=GAN_output)
    # GAN.compile(loss='binary_crossentropy', optimizer=Adam(0.0002, 0.5))
    GAN.compile(loss='binary_crossentropy', optimizer=Adam(0.0002, 0.5))
    return GAN


#传入序列是数据框格式，（label,seq)
def train_GAN(X_train1,epochs=200, batch_size=128,seq_len=201):
    X_train = one_hot(X_train1.iloc[:, 1].values)
    X_train = X_train.reshape(-1, seq_len*4)
    # Loading the data
    # Creating GAN
    generator = build_generator()
    discriminator = build_discriminator()
    GAN = build_GAN(discriminator, generator)

    for i in range(1, epochs + 1):
        # print("\n","Epoch %d" % i)

        for _ in tqdm(range(batch_size)):
            # Generate fake images from random noiset
            noise = np.random.normal(0, 1, (batch_size, 100))
            fake_images = generator.predict(noise)

            # Select a random batch of real images from MNIST
            real_images = X_train[np.random.randint(0, X_train.shape[0], batch_size)]

            # Labels for fake and real images
            # label_fake = np.zeros(batch_size)
            # label_real = np.ones(batch_size)
            label_fake = np.random.uniform(0, 0.2, size=batch_size)
            label_real = np.random.uniform(0.8, 1.0, size=batch_size)

            # Concatenate fake and real images
            X = np.concatenate([fake_images, real_images])
            y = np.concatenate([label_fake, label_real])

            # Train the discriminator
            discriminator.trainable = True
            discriminator.train_on_batch(X, y)

            # Train the generator/chained GAN model (with frozen weights in discriminator)
            discriminator.trainable = False
            GAN.train_on_batch(noise, label_real)

        # Draw generated images every 15 epoches
        # if i == 0 or i % 10 == 0:
        #     generate_positive_for_plot(generator,X_train1, i,dataset, examples=5000,save_model=True,seq_len=201,aug=True)

    #     # draw_images(generator, i)
        #     generate_positive(generator, i,10000)
    return generator


#输入X_train为纯正样本
def generate_positive(X_train, epoch,dataset, examples=10000,save_model=True,seq_len=201,aug=True):

    if aug:
        print("翻转前的X_train：", X_train.shape)
        seq_temp = X_train["seq"].tolist()
        seq_temp2 = [seq1[::-1] for seq1 in seq_temp]
        label_temp = [1]*len(seq_temp2)
        df_temp = pd.DataFrame({"label":label_temp,"seq":seq_temp2})
        X_train = pd.concat([X_train,df_temp],axis=0)
        print("翻转后的X_train：",X_train.shape)

    generator = train_GAN(X_train, epochs=epoch, batch_size=128)
    if save_model:
        generator.save_weights("result/weight/generator" + dataset + ".w")

    noise= np.random.normal(loc=0, scale=1, size=[examples, 100])
    generated_images = generator.predict(noise)
    generated_images = generated_images.reshape(-1,seq_len,4)
    DNAs = []
    for i in range(examples):
        DNA1 = generated_images[i]
        max_indices = np.argmax(DNA1, axis=1)
        DNA = [dict1[x] for x in max_indices]
        DNA = "".join(DNA)
        DNAs.append(DNA)
    labels = [1]*len(DNAs) #需要注意到是，训练的时候fake是0，后面用于预训练的时候是1
    DNAs = pd.DataFrame({"label":labels,"seq":DNAs})
    # DNAs.to_csv("result/fake_DNAs_"+dataset_name+str(epoch)+"_2.csv")
    return DNAs

#输入X_train为纯正样本
def generate_negative(X_train, epoch,dataset, examples=10000,save_model=True,seq_len=201,aug=True):
    if aug:
        print("翻转前的X_train：", X_train.shape)
        seq_temp = X_train["seq"].tolist()
        seq_temp2 = [seq1[::-1] for seq1 in seq_temp]
        label_temp = [1]*len(seq_temp2)
        df_temp = pd.DataFrame({"label":label_temp,"seq":seq_temp2})
        X_train = pd.concat([X_train,df_temp],axis=0)
        print("翻转后的X_train：",X_train.shape)

    generator = train_GAN(X_train, epochs=epoch, batch_size=128)
    if save_model:
        generator.save_weights("result/weight/generator" + dataset + ".w")

    noise= np.random.normal(loc=0, scale=1, size=[examples, 100])
    generated_images = generator.predict(noise)
    generated_images = generated_images.reshape(-1,seq_len,4)
    DNAs = []
    for i in range(examples):
        DNA1 = generated_images[i]
        max_indices = np.argmax(DNA1, axis=1)
        DNA = [dict1[x] for x in max_indices]
        DNA = "".join(DNA)
        DNAs.append(DNA)
    labels = [0]*len(DNAs) #需要注意到是，训练的时候fake是0，后面用于预训练的时候是1
    DNAs = pd.DataFrame({"label":labels,"seq":DNAs})
    # DNAs.to_csv("result/fake_DNAs_"+dataset_name+str(epoch)+"_2.csv")
    return DNAs


def evaluate2(dataset,epoch=200,the=0.8,num_total=300000,sample_times=20,seq_len=201):
    train_X_org, test_X_org = get_single_data(dataset)
    # X_train_org, _, _ = ml_code(train_X_org, "training", {"ENAC2": ENAC2})
    X_train_org, _, _ = ml_code(train_X_org, "training", {"binary": binary})
    # X_test_org, _, _ = ml_code(test_X_org, "training", {"ENAC2": ENAC2})
    X_test_org, _, _ = ml_code(test_X_org, "training", {"binary": binary})
    y_train_org = get_label(train_X_org)
    y_test_org = get_label(test_X_org)
    model = LGBMClassifier()
    model.fit(X_train_org, y_train_org)
    y_pred_LGBM = model.predict_proba(X_test_org)[:, 1]
    print("LGBM自己的测试集效果：", score2(y_test_org, y_pred_LGBM))
    # ===================================================================================
    X_train_pos = train_X_org.iloc[y_train_org == 1, :]
    X_train_neg = train_X_org.iloc[y_train_org == 0, :]
    train_X_fake_pos = generate_positive(X_train_pos, epoch, dataset, num_total,True,seq_len)
    train_X_fake_neg = generate_negative(X_train_neg, epoch, dataset, num_total, True, seq_len)



    # data_fake_pos = train_X_fake_pos.iloc[:, 1:]
    # X_fake, y_fake, record_feature_type = ml_code(train_X_fake_pos, "training", {"ENAC2": ENAC2})
    X_fake_pos, y_fake_pos, record_feature_type = ml_code(train_X_fake_pos, "training", {"binary": binary})
    # X_fake_neg, y_fake_neg, record_feature_type = ml_code(train_X_fake_pos, "training", {"binary": binary})
    X_fake_neg, y_fake_neg, record_feature_type = ml_code(train_X_fake_neg, "training", {"binary": binary})

    y_pred_pos = model.predict_proba(X_fake_pos)[:, 1]
    data_fake_pos2 = train_X_fake_pos.iloc[y_pred_pos > the, :]
    print("筛选到的正样本数：",data_fake_pos2.shape[0],train_X_fake_pos.shape[0])

    y_pred_neg = model.predict_proba(X_fake_neg)[:, 1]
    # data_fake_neg2 = train_X_fake_neg.iloc[y_pred_neg < (1-the), :]
    data_fake_neg2 = train_X_fake_neg.iloc[y_pred_neg < 0.5, :]
    print("筛选到的负样本数：", data_fake_neg2.shape[0],train_X_fake_neg.shape[0])
    #-------------------------------------------------------------------

    pos_len = data_fake_pos2.shape[0]
    neg_len = data_fake_neg2.shape[0]

    len1 = min(pos_len,neg_len)


    index_1= list(range(len1))
    y = [0] * len1 + [1] * len1
    # y = [0] * neg_len + [1] * neg_len
    # y2 = [3] * neg_len + [4] * neg_len
    data_save = []
    result_trains = []
    result_tests = []
    data_plot = []
    for i in range(sample_times):
        index_pos_choosed = random.sample(range(pos_len), len1)
        index_neg_choosed = random.sample(range(neg_len), len1)
        data_pos_temp = X_fake_pos[index_pos_choosed]
        data_neg_temp = X_fake_neg[index_neg_choosed]
        data = np.vstack([data_neg_temp, data_pos_temp])
        model2 = LGBMClassifier()
        model2.fit(data, y)
        data_plot.append([data,y])

        y_pred_LGBM = model2.predict_proba(X_train_org)[:, 1]
        result_train = score2(y_train_org, y_pred_LGBM)
        print("第" + str(i) + "轮训练集：", result_train)

        y_pred_LGBM = model2.predict_proba(X_test_org)[:, 1]
        result_test = score2(y_test_org, y_pred_LGBM)
        print("第"+str(i)+"轮测试集：", result_test)

        result_trains.append(result_train)
        result_tests.append(result_test)


        data_save.append(pd.concat([train_X_fake_pos.iloc[index_pos_choosed,:],train_X_fake_neg.iloc[index_pos_choosed,:]],axis=0))

    with open("result/fake_data_bn.pkl","wb") as f:
        pkl.dump(data_save,f)

    result_trains = np.array(result_trains)
    result_tests = np.array(result_tests)

    print("training summary...")
    print(result_trains.mean(axis=0))
    print(result_trains.max(axis=0))
    print(result_trains.min(axis=0))

    print("testing summary...")
    print(result_tests.mean(axis=0))
    print(result_tests.max(axis=0))
    print(result_tests.min(axis=0))





if __name__ == "__main__":
    start = time.time()
    SEED = 0
    set_global_determinism(SEED)
    dataset = "ac4c"
    # # evaluate1(dataset, epoch=400, the=0.80, num_total=300000, sample_times=20,seq_len=201)
    # evaluate2(dataset, epoch=10, the=0.80, num_total=300000, sample_times=20,seq_len=201)
    #
    # end = time.time()
    # print("Time consuming:%.2f秒" % (end - start))

    #
    evaluate2(dataset, epoch=30, the=0.80, num_total=10000, sample_times=20, seq_len=201)