from tensorflow.python.keras import Sequential, Input, Model
from tensorflow.python.keras.layers import Dense, LeakyReLU, BatchNormalization, Reshape
import numpy as np

def build_generator(self):
    # 使用Sequential方式构建生成器

    model = Sequential()

    # 输入为100维的随机噪声数据，经过第一层全连接层后输出为256维
    model.add(Dense(256, input_dim=self.latent_dim))
    model.add(LeakyReLU(alpha=0.2))
    model.add(BatchNormalization(momentum=0.8))

    model.add(Dense(512))  # 经过第二层全连接层，输出为512维
    model.add(LeakyReLU(alpha=0.2))
    model.add(BatchNormalization(momentum=0.8))

    model.add(Dense(1024))  # 经过第三层全连接层，输出为1024维
    model.add(LeakyReLU(alpha=0.2))
    model.add(BatchNormalization(momentum=0.8))

    # 经过第四层全连接层，输出为设置的图片的尺寸（这里是28*28，也就是784维）
    model.add(Dense(np.prod(self.img_shape), activation='tanh'))
    model.add(Reshape(self.img_shape))  # Reshape成1*28*28

    model.summary()  # 打印网络结构

    # 下面两句就是给定输入，获取一个输出
    # 有点类似pytorch模型类中forward函数的意思
    noise = Input(shape=(self.latent_dim,))
    img = model(noise)
    return Model(noise, img)  # 返回构造好的模型